/*
 * AES encryption library.
 *
 * Velciu Veronica-Mihaela
 * 2017
 */

#include "AES.h"

void encryptECB(uint8_t *key, uint8_t *plaintext, uint8_t len, uint8_t *ciphertext)
{
    uint8_t i, j = 0;
    
    /* Key setup */
    for (i = 0; i < 16; i++)
    {
        AES_KEY_struct.aes_key = key[i];
    }
    
    /* AES ECB encryption configuration, no interrupt */
    AES_CTRL_struct.aes_im = 0;
    AES_CTRL_struct.aes_dir = 0;
    AES_CTRL_struct.aes_mode = 0;
    
    while (j + 16 <= len)
    {
        /* Write data */
        for (i = 0; i < 16; i++)
        {
            AES_STATE_struct.aes_state = plaintext[j + i];
        }
        
        /* Start operation */
        AES_CTRL_struct.aes_request = 1;
        
        /* Wait for AES finished */
        while (!AES_STATUS_struct.aes_done);
        
        /* Read cipher text */
        for (i = 0; i < 16; i++)
        {
            ciphertext[j + i] = AES_STATE_struct.aes_state;
        }
        
        j += 16;
    }
}

void decryptECB(uint8_t *key, uint8_t *ciphertext, uint8_t len, uint8_t *plaintext)
{
    uint8_t i, j = 0;
    uint8_t round_key[16];
    
    /* Run a dummy encryption using the key */
    encryptECB(key, round_key, 16, round_key);
    /* Get the round key */
    for (i = 0; i < 16; i++)
    {
        round_key[i] = AES_KEY_struct.aes_key;
    }
    
    /* Key setup (the round key must be used insted of the original one) */
    for (i = 0; i < 16; i++)
    {
        AES_KEY_struct.aes_key = round_key[i];
    }
    
    /* AES ECB decryption configuration, no interrupt */
    AES_CTRL_struct.aes_im = 0;
    AES_CTRL_struct.aes_dir = 1;
    AES_CTRL_struct.aes_mode = 0;
    
    while (j + 16 <= len)
    {
        /* Write data */
        for (i = 0; i < 16; i++)
        {
            AES_STATE_struct.aes_state = ciphertext[j + i];
        }
        
        /* Start operation */
        AES_CTRL_struct.aes_request = 1;
        
        /* Wait for AES finished */
        while (!AES_STATUS_struct.aes_done);
        
        /* Read plain text */
        for (i = 0; i < 16; i++)
        {
            plaintext[j + i] = AES_STATE_struct.aes_state;
        }
        
        j += 16;
    }
}

void encryptCBC(uint8_t *key, uint8_t *iv, uint8_t *plaintext, uint8_t len, uint8_t *ciphertext)
{
    uint8_t i, j = 0;
    uint8_t block1[16];
    
    for (i = 0; i < 16; i++)
    {
        block1[i] = plaintext[i] ^ iv[i];
    }
    
    /* The first CBC run has to be configured as ECB to process the initial data (plain text XOR IV) */
    encryptECB(key, block1, 16, ciphertext);
    j += 16;
    
    /* Change configuration to CBC */
    AES_CTRL_struct.aes_mode = 1;
	
	/* Compute the rest of the blocks */
    while (j + 16 <= len)
    {
        /* Write data */
        for (i = 0; i < 16; i++)
        {
            AES_STATE_struct.aes_state = plaintext[j + i];
        }
        
        /* Start operation */
        AES_CTRL_struct.aes_request = 1;
        
        /* Wait for AES finished */
        while (!AES_STATUS_struct.aes_done);
        
        /* Read cipher text */
        for (i = 0; i < 16; i++)
        {
            ciphertext[j + i] = AES_STATE_struct.aes_state;
        }
        
        j += 16;
    }
}

